#include "Bildschirm.h"
#include "Punkt.h"
#include "Bild.h"
#include "Farbe.h"
#include "Fenster.h"
#include "Text.h"
#include "Objekt.h"

using namespace Framework;

// Inhalt der Bildschirmklass aus Bildschirm.h
// Konstruktor 
Bildschirm::Bildschirm( WFenster *f )
{
	fenster = f;
	pDirect3D = 0;
	pDevice = 0;
	pBackBuffer = 0;
	renderB = new Bild();
	ref = 1;
	InitializeCriticalSection( &threadSave );
	members = new ObjektArray();
	fllFarbe = new Farbe( 0, 0, 0, 0 );
	vollbild = 0;
	backBufferGre = 0;
}

// Destruktor 
Bildschirm::~Bildschirm()
{
	if( renderB )
		renderB->release();
	renderB = 0;
	if( pDevice )
	{
		pDevice->Release();
		pDevice = NULL;
	}
	if( pDirect3D )
	{
		pDirect3D->Release();
		pDirect3D = NULL;
	}
	if( pBackBuffer )
	{
		pBackBuffer->Release();
		pBackBuffer = NULL;
	}
	if( fenster )
		fenster->release();
	if( fllFarbe )
		fllFarbe->release();
	if( backBufferGre )
		backBufferGre->release();
	DeleteCriticalSection( &threadSave );
	delete members;
}

// nicht konstant 
void Bildschirm::lock()
{
	EnterCriticalSection( &threadSave );
}

void Bildschirm::unlock()
{
	LeaveCriticalSection( &threadSave );
}

void Bildschirm::update()
{
	lock();
	HRESULT result;

	backRect.pBits = NULL;
	if( pDirect3D )
	{
		pDirect3D->Release();
		pDirect3D = 0;
	}
	if( pDevice )
	{
		pDevice->Release();
		pDevice = 0;
	}
	if( pBackBuffer )
	{
		pBackBuffer->Release();
		pBackBuffer = 0;
	}
	pDirect3D = Direct3DCreate9( D3D_SDK_VERSION );

    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp,sizeof( d3dpp ) );
	d3dpp.Windowed = !vollbild;
	d3dpp.hDeviceWindow = fenster->getFensterHandle();
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = D3DFMT_X8R8G8B8;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_ONE;
	d3dpp.Flags = D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;
	Punkt *gre;
	if( !backBufferGre )
		gre = fenster->getGre();
	else
		gre = backBufferGre->getThis();
	d3dpp.BackBufferHeight = gre->getY();
	d3dpp.BackBufferWidth = gre->getX();
	if( renderB )
		renderB->release();
	renderB = new Bild();
	if( !fllFarbe )
		renderB->neuBild( gre, new Farbe() );
	else
		renderB->neuBild( gre, fllFarbe->getThis() );

    result = pDirect3D->CreateDevice( D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,fenster->getFensterHandle(),
		D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_PUREDEVICE,&d3dpp,&pDevice );

	result = pDevice->GetBackBuffer( 0,0,D3DBACKBUFFER_TYPE_MONO,&pBackBuffer );
	unlock();
}

void Bildschirm::addMember( Objekt *obj ) // Fgt ein Objekt hinzu
{
	lock();
	members->addObjekt( obj );
	members->updateIndex( 0 );
	unlock();
}

void Bildschirm::removeMember( Objekt *obj ) // Entfernt ein Objekt
{
	lock();
	members->removeObjekt( obj );
	members->updateIndex( 0 );
	unlock();
}

void Bildschirm::render()
{
	int count = 0;
	if( renderB && pDevice )
	{
		lock();
		if( fllFarbe ) // clear screen
			renderB->fllRegion( 0, 0, renderB->getBreite(), renderB->getHhe(), fllFarbe->getFarbe() );
		members->render( renderB ); // zeichnen nach zwischenbuffer
		// Beginne Bild 
		HRESULT result;
		result = pDevice->Clear( 0,NULL,D3DCLEAR_TARGET,D3DCOLOR_XRGB(0,0,0),0.0f,0 );
		result = pBackBuffer->LockRect( &backRect, 0, 0 );
		// kopieren zum Bildschrirm 
		for(int y = 0; y < renderB->getHhe(); y++)
		{
			memcpy( &( ( BYTE *)backRect.pBits )[backRect.Pitch * y], (void*)&( ( renderB->getBuffer() )[renderB->getBreite() * y] ), sizeof( D3DCOLOR ) * renderB->getBreite() );
		}
		// Beende Bild 
		result = pBackBuffer->UnlockRect();
		result = pDevice->Present( 0, 0, 0, 0 );
		if( result != S_OK )
		{
			count++;
			update();
		}
		else if( count )
			count--;
		unlock();
	}
	if( !pDevice )
	{
		count++;
		update();
	}
	if( count > 10 )
	{
		WMessageBox( fenster ? fenster->getFensterHandle() : 0, new Text( "Fehler" ), new Text( "Es int ein Fehler beim rendern." ), MB_ICONERROR );
		count = 0;
	}
}

void Bildschirm::setFllFarbeZ( Farbe *f ) // setzt die Fill Farbe
{
	if( fllFarbe )
		fllFarbe->release();
	fllFarbe = f;
}

void Bildschirm::setFllFarbe( Farbe *f )
{
	if( !fllFarbe )
		fllFarbe = new Farbe();
	fllFarbe->setFarbe( f->getFarbe() );
}

void Bildschirm::setFllFarbe( int f )
{
	if( !fllFarbe )
		fllFarbe = new Farbe();
	fllFarbe->setFarbe( f );
}

void Bildschirm::setVollbild( bool vollbild ) // setzt vollbild
{
	this->vollbild = vollbild;
}

void Bildschirm::tick( double tickval )
{
	members->tick( tickval );
}

void Bildschirm::setBackBufferGre( int breite, int hhe ) // setzt die Gre des Backbuffers
{
	if( !backBufferGre )
		backBufferGre = new Punkt();
	backBufferGre->setP( breite, hhe );
}

void Bildschirm::setBackBufferGre( Punkt *gre )
{
	if( !backBufferGre )
		backBufferGre = new Punkt();
	backBufferGre->setP( gre->x, gre->y );
	gre->release();
}

void Bildschirm::setBackBufferGreZ( Punkt *gre )
{
	if( backBufferGre )
		backBufferGre->release();
	backBufferGre = gre;
}

// constant 
Bild *Bildschirm::getRenderBild() const
{
	return renderB->getThis();
}

ObjektArray *Bildschirm::getMembers() const // gibt die Objekte zurck
{
	return members;
}

Farbe *Bildschirm::getFllFarbe() const // gibt die Fll Farbe zurck
{
	return fllFarbe ? fllFarbe->getThis(): 0;
}

Farbe *Bildschirm::zFllFarbe() const
{
	return fllFarbe;
}

bool Bildschirm::istVolbild() const // gibt zurck, ob vollbild an ist
{
	return vollbild;
}

Punkt *Bildschirm::getBackBufferGre() const // gibt die Gre des Backbuffers zurck
{
	if( backBufferGre )
		return backBufferGre->getThis();
	return 0;
}

Punkt *Bildschirm::zBackBufferGre() const
{
	return backBufferGre;
}

// Reference Counting 
Bildschirm *Bildschirm::getThis()
{
	ref++;
	return this;
}

Bildschirm *Bildschirm::release()
{
	ref--;
	if( !ref )
		delete this;
	return 0;
}