#ifndef Bildschirm_H
#define Bildschirm_H

#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#include <d3d9.h>

namespace Framework
{
	class Bild; // Bild.h
	class Punkt; // Punkt.h
	class WFenster; // Fenster.h
	class Bildschirm; // aus dieser Datei
	class ObjektArray; // Objekt.h
	class Objekt; // Objekt.h
	class Farbe; // Farbe.h

	//Diese Klasse wird das Bild auf dem Bildschirm verwalten
	class Bildschirm
	{
	private:
		IDirect3D9*			pDirect3D;
		IDirect3DDevice9*	pDevice;
		IDirect3DSurface9*	pBackBuffer;
		D3DLOCKED_RECT		backRect;
		WFenster *fenster;
		Bild *renderB;
		CRITICAL_SECTION threadSave;
		int ref;
		ObjektArray *members;
		Farbe *fllFarbe;
		bool vollbild;
		Punkt *backBufferGre;

	public:
		// Konstruktor 
		Bildschirm( WFenster *fenster );
		// Destruktor 
		~Bildschirm();
		// nicht constant 
		void lock();
		void unlock();
		void update(); // aktualisiert directX
		void addMember( Objekt *obj ); // Fgt ein Objekt hinzu
		void removeMember( Objekt *obj ); // Entfernt ein Objekt
		void render(); // Zeichnet das Bild
		void setFllFarbeZ( Farbe *f ); // setzt die Fill Farbe
		void setFllFarbe( Farbe *f );
		void setFllFarbe( int f );
		void setVollbild( bool fullscreen ); // setzt vollbild
		void tick( double tickval );
		void setBackBufferGre( int breite, int hhe ); // setzt die Gre des Backbuffers
		void setBackBufferGre( Punkt *gre );
		void setBackBufferGreZ( Punkt *gre );
		// constant 
		Bild *getRenderBild() const; // Gibt das RenderObj zurck
		ObjektArray *getMembers() const; // gibt die Objekte zurck
		Farbe *getFllFarbe() const; // gibt die Fll Farbe zurck
		Farbe *zFllFarbe() const;
		bool istVolbild() const; // gibt zurck, ob vollbild an ist
		Punkt *getBackBufferGre() const; // gibt die Gre des Backbuffers zurck
		Punkt *zBackBufferGre() const;
		// Reference Counting 
		Bildschirm *getThis();
		Bildschirm *release();
	};
}

#endif