//---Include---
#include "DateiSystem.h"
#include "Bild.h"
#include "Text.h"
#include "Farbe.h"
#include "Punkt.h"
#include <Windows.h>
#include <iostream>
#include "Fenster.h"
#include "Schrift.h"
#include "Datei.h"
#include "Fortschritt.h"

using namespace Framework;

// LTDS Dateivormat 
// Inhalt der LTDSPixel Klasse aus DateiSystem.h
// Konstruktor 
LTDSPixel::LTDSPixel( LTDSPixel *davor )
{
	this->davor = davor;
	index = 0, maxIndex = 1;
	nder = 0, nderA = 0, komp = 0;
	iA = 0;
	miA = 8;
	alpha = 0;
	ref = 1;
}

// Destruktor 
LTDSPixel::~LTDSPixel()
{
	if( davor )
		davor->release();
}

// nicht constant 
// zum Laden gedacht 
bool LTDSPixel::addBitZuFarbe( unsigned char bit )
{
	if( nderA && iA != miA ) // Das Bit gehrt zu Alpha
	{
		alpha |= ( ( bit & Bits( 1 ) ) ) << ( 7 - komp - iA );
		iA++;
	}
	else // Das Bit gehrt zum nchsten Pixel
		return false;
	return true;
}

char LTDSPixel::addByte( char byte, char begin ) // gibt ende des Pixels zurck, -1 wenn nicht zu ende
{
	if( begin >= 8 || begin < 0 )
		return -1;
	for( int i = begin; i < 8; i++ )
	{
		switch( index )
		{
		case 0:
			// Das erste Bit eines Pixels speichert, ob sich an der Komprimierung etwas nderte
			nder = ( ( byte >> ( 7 - i ) ) & Bits( 1 ) ) == 1;
			if( !nder ) // ndert sich nichts an der Komprimierung, so werden die Werte vom vorherigen Pixel bernommen
			{
				if( !davor ) // Die Datei ist beschdigt ( Der erste Pixel kann nicht von dem davor bernemen )
				{
					MessageBox( NULL, "Fehler, die Bilddatei ist beschdigt", "Fehler", MB_ICONERROR );
					exit( 0 );
				}
				nderA = davor->getnderA();
				komp = davor->getKomp();
				miA -= komp;
				if( !nderA )
					alpha = davor->getA();
				maxIndex += nderA * ( 8 - komp ); // Bestimmung der Lnge
				// des Pixels in Bits. Jede Farbe hat von grund auf 8 Bits, durch komprimierung kanns kleiner sein
			}
			else
				maxIndex += 4; // Da der Pixel nicht die Werte des vorherigen bernimmt, wird er um 6 Bits grer
			break;
		case 1: // Das zweite Bit eines Pixels speichert entweder die Komprimierungsart, oder schon die Farbe
			if( nder ) // Das Bit gehrt zu den 7 Komprimierungsbits
				nderA = ( ( byte >> ( 7 - i ) ) & Bits( 1 ) ) == 1;
			else
			{
				if( !addBitZuFarbe( byte >> ( 7 - i ) ) ) // Das Bit gehrt zur Farbe
					return i;
			}
			break;
		case 2: // Das sechste Bit eines Pixels speichert entweder die Komprimierungsart, oder schon die Farbe
			if( nder ) // Das Bit gehrt zu den 7 Komprimierungsbits
				komp |= (unsigned char)( ( byte >> ( 7 - i ) ) & Bits( 1 ) ) << 2;
			else
			{
				if( !addBitZuFarbe( byte >> ( 7 - i ) ) ) // Das Bit gehrt zur Farbe
					return i;
			}
			break;
		case 3: // Das siebte Bit eines Pixels speichert entweder die Komprimierungsart, oder schon die Farbe
			if( nder ) // Das Bit gehrt zu den 7 Komprimierungsbits
				komp |= (unsigned char)( ( byte >> ( 7 - i ) ) & Bits( 1 ) ) << 1;
			else
			{
				if( !addBitZuFarbe( byte >> ( 7 - i ) ) ) // Das Bit gehrt zur Farbe
					return i;
			}
			break;
		case 4: // Das achte Bit eines Pixels speichert entweder die Komprimierungsart, oder schon die Farbe
			if( nder ) // Das Bit gehrt zu den 7 Komprimierungsbits
			{
				komp |= (unsigned char)( ( byte >> ( 7 - i ) ) & Bits( 1 ) );
				// Das war das letzte Komprimierungsbit
				// Komprimierung auswerten 
				miA -= komp;
				if( !nderA )
					alpha = davor->getA();
				maxIndex += nderA * ( 8 - komp ); // Bitlnge des Pixels
			}
			else
			{
				if( !addBitZuFarbe( byte >> ( 7 - i ) ) ) // Das Bit gehrt zur Farbe
					return i;
			}
			break;
		default: // Die restlichen Bits speichern alle die Farbwerte des Pixels
			if( !addBitZuFarbe( byte >> ( 7 - i ) ) ) // Das Bit gehrt zur Farbe
				return i;
			break;
		}
		index++;
		if( index >= maxIndex )
		{
			if( davor )
			{
				if( nderA )
					alpha = davor->getA() + alpha;
				davor = davor->release();
			}
			return i + 1;
		}
	}
	return -1;
}

// zum speichern gedacht 
void LTDSPixel::setAlpha( unsigned char alpha ) // setzt die Farbe des Pixels
{
	this->alpha = alpha;
}

void LTDSPixel::Komp() // Komprimiert den Pixel
{
	maxIndex = 1;
	if( !davor )
	{// Das ist der erste Pixel
		nder = 1;
		nderA = 1;
		maxIndex += 4;
		miA = getBits( alpha );
	}
	else
	{ // Es wird die differenz zum vorrigen Pixel gespeichert
		miA = getBits( alpha - davor->getA() );
		if( alpha != davor->getA() )
			nderA = 1;
		else
			nderA = 0;
	}// Prfen ob sich etwas ndert
	if( !miA && nderA )
		miA++;
	komp = 8 - miA;
	maxIndex += nderA * miA;
	if( davor )
	{
		if( nderA != davor->getnderA() ||
			komp != davor->getKomp() )
		{ // Es ndert sich etwas
			nder = 1;
			maxIndex += 4;
		}
		else
		{ // Es ndert sich nichts
			nder = 0;
		}
	}
}

bool LTDSPixel::getNextFarbeBit( char &byte, int i )
{
	unsigned char AA = alpha;
	if( davor )
	{
		AA -= davor->getA();
	}
	if( nderA && iA != miA ) // Das Bit gehrt zu Alpha
	{
		byte |= ( ( AA >> ( 7 - komp - iA ) ) & Bits( 1 ) ) << ( 7 - i );
		iA++;
	}
	else // Der Pixel ist bereits zu ende
		return false;
	return true;
}

char LTDSPixel::getNextByte( char &byte, int bbegin ) // Gibt die nchsten Bits Zurck, -1 wenn der Pixel nicht zu ende ist
{
	// bbegin gibt an wohin in die byte-variable geschrieben werden soll
	// die Funktion gibt das ende des Pixels in der byte-variable zurck
	// -1 heit, dass der Pixel nicht zu ende ist
	for( int i = bbegin; i < 8; i++ )
	{
		switch( index )
		{
		case 0: // Das erste Bit des Pixels speichert, ob sich etwas an der Komprimierung ndert
			byte |= ( (int)nder & Bits( 1 ) ) << ( 7 - i );
			break;
		case 1: // Das zweite Bit des Pixels speichert entweder schon die Farbe oder die Komprimierung
			if( nder ) // Komprimierung
				byte |= ( (int)nderA & Bits( 1 ) ) << ( 7 - i );
			else // Farbe
				if( !getNextFarbeBit( byte, i ) )
					return i;
			break;
		case 2: // Das zweite Bit des Pixels speichert entweder schon die Farbe oder die Komprimierung
			if( nder ) // Komprimierung
				byte |= ( ( komp >> 2 ) & Bits( 1 ) ) << ( 7 - i );
			else // Farbe
				if( !getNextFarbeBit( byte, i ) )
					return i;
			break;
		case 3: // Das dritte Bit des Pixels speichert entweder schon die Farbe oder die Komprimierung
			if( nder ) // Komprimierung
				byte |= ( ( komp >>1 ) & Bits( 1 ) ) << ( 7 - i );
			else // Farbe
				if( !getNextFarbeBit( byte, i ) )
					return i;
			break;
		case 4: // Das vierte Bit des Pixels speichert entweder schon die Farbe oder die Komprimierung
			if( nder ) // Komprimierung
				byte |= ( komp & Bits( 1 ) ) << ( 7 - i );
			else // Farbe
				if( !getNextFarbeBit( byte, i ) )
					return i;
			break;
		default: // Die restlichen Bits speichern die Farbe des Pixels
			if( !getNextFarbeBit( byte, i ) )
				return i;
			break;
		}
		index++;
		if( index >= maxIndex )
		{
			if( davor )
				davor = davor->release();
			return i + 1;
		}
	}
	return -1;
}

// constant 
unsigned char LTDSPixel::getKomp() const // hat sich die Komprimierung gendert
{
	return komp;
}

bool LTDSPixel::getnderA() const // gibt zurck, ob sich der alphawert ndert
{
	return nderA;
}

unsigned char LTDSPixel::getA() const // gibt Alpha zurck
{
	return alpha;
}

// Reference Counting 
LTDSPixel *LTDSPixel::getThis()
{
	ref++;
	return this;
}

LTDSPixel *LTDSPixel::release()
{
	ref--;
	if( ref == 0 )
		delete this;
	return 0;
}

// Inhalt der LTDSDateiKopf Klasse aus Dateisystem.h
// Konstruktor 
LTDSDateiKopf::LTDSDateiKopf()
{
	ref = 1;
	sganzahl = 0;
	gr = 0;
	pos = 0;
}

// Destruktor 
LTDSDateiKopf::~LTDSDateiKopf()
{
	delete []gr;
	delete []pos;
}

// nicht constant 
void LTDSDateiKopf::laden( std::ifstream *inF ) // Lt aus inF
{
	if( inF->good() && inF->is_open() )
	{
		inF->read( (char *)&sganzahl, 1 );
		if( sganzahl > 0 )
		{
			delete []gr;
			delete []pos;
			gr = new unsigned char[ sganzahl + 1 ];
			gr[ sganzahl ] = '\0';
			pos = new int[ sganzahl ];
		}
		for( int i = 0; i < sganzahl; i++ )
		{
			inF->read( (char*)&gr[ i ], 1 );
			inF->read( (char*)&pos[ i ], 4 );
		}
	}
}

void LTDSDateiKopf::addSG( char sg ) // Schriftgre hinzufgen
{
	sganzahl++;
	unsigned char *gr_tmp = gr;
	int *pos_tmp = pos;
	gr = new unsigned char[ sganzahl ];
	pos = new int[ sganzahl ];
	if( sganzahl - 1 > 0 )
	{
		memcpy( gr, gr_tmp, 1 * ( sganzahl - 1 ) );
		memcpy( pos, pos_tmp, 4 * ( sganzahl - 1 ) );
	}
	delete []gr_tmp;
	delete []pos_tmp;
	pos[ sganzahl - 1 ] = 0;
	gr[ sganzahl - 1 ] = sg;
}

void LTDSDateiKopf::removeSG( char sg ) // Schriftgre entfernen
{
	bool hatsg = 0;
	int sgpos = 0;
	for( int i = 0; i < sganzahl; i++ )
	{
		hatsg = gr[ i ] == sg;
		sgpos = i;
		if( hatsg )
			break;
	}
	if( hatsg )
	{
		sganzahl--;
		unsigned char *gr_tmp = gr;
		int *pos_tmp = pos;
		gr = new unsigned char[ sganzahl ];
		pos = new int[ sganzahl ];
		for( int i = 0; i < sgpos; i++ )
		{
			gr[ i ] = gr_tmp[ i ];
			pos[ i ] = pos_tmp[ i ];
		}
		for( int i = sgpos + 1; i < sganzahl; i++ )
		{
			gr[ i - 1 ] = gr_tmp[ i ];
			pos[ i - 1 ] = pos_tmp[ i ];
		}
		delete []gr_tmp;
		delete []pos_tmp;
	}
}

// constant 
void LTDSDateiKopf::speichern( std::ofstream *outF ) const // Speichert nach outF
{
	if( outF->is_open() && outF->good() )
	{
		outF->write( (char*)&sganzahl, 1 );
		for( int i = 0; i < sganzahl; i++ )
		{
			outF->write( (char*)&gr[ i ], 1 );
			outF->write( (char*)&pos[ i ], 4 );
		}
	}
}

unsigned char *LTDSDateiKopf::getSchriftGreList() const // gibt eine Liste mit gespeicherten Schriftgren zurck
{
	return gr;
}

int *LTDSDateiKopf::getPositionList() const // gibt eine Positionsliste der gespeicherten Schriftgren zurck
{
	return pos;
}

int LTDSDateiKopf::getSchriftGreAnzahl() const // gibt die Anzahl der gespeicherten Schriftgren zurck
{
	return sganzahl;
}

// Reference Counting 
LTDSDateiKopf *LTDSDateiKopf::getThis()
{
	ref++;
	return this;
}

LTDSDateiKopf *LTDSDateiKopf::release()
{
	ref--;
	if( ref == 0 )
		delete this;
	return 0;
}

// Inhalt der LTDSSchriftKopf Klasse aus Dateisystem.h
// Konstruktor 
LTDSSchriftKopf::LTDSSchriftKopf()
{
	ref = 1;
	schriftGre = 0;
	zeichen = 0;
	pos = 0;
	zeichenAnzahl = 0;
}

// Destruktor 
LTDSSchriftKopf::~LTDSSchriftKopf()
{
	delete []pos;
	delete []zeichen;
}

// nicht constant 
void LTDSSchriftKopf::laden( std::ifstream *inF ) // lht von inF
{
	if( inF->good() && inF->is_open() )
	{
		inF->read( (char*)&schriftGre, 1 );
		inF->read( (char*)&zeichenAnzahl, 1 );
		delete []pos;
		delete []zeichen;
		zeichen = new unsigned char[ zeichenAnzahl ];
		pos = new int[ zeichenAnzahl ];
		for( int i = 0; i < zeichenAnzahl; i++ )
		{
			inF->read( (char*)&zeichen[ i ], 1 );
			inF->read( (char*)&pos[ i ], 4 );
		}
	}
}

void LTDSSchriftKopf::setSchriftgre( unsigned char gr ) // setze schriftgre
{
	schriftGre = gr;
}

void LTDSSchriftKopf::setZeichenAlphabet( Alphabet *alphabet ) // setzt die Zeichen von alphabet
{
	int count = 0;
	for( int i = 0; i < 256; i++ )
	{
		Buchstabe *zeich = alphabet->getBuchstabe( i );
		if( zeich )
			count++;
	}
	delete []zeichen;
	delete []pos;
	zeichen = new unsigned char[ count ];
	pos = new int[ count ];
	count = 0;
	for( int i = 0; i < 256; i++ )
	{
		Buchstabe *zeich = alphabet->getBuchstabe( i );
		if( zeich )
		{
			zeichen[ count ] = i;
			pos[ count ] = 0;
			count++;
			zeich->release();
		}
	}
	schriftGre = alphabet->getSchriftgre();
	alphabet->release();
}

void LTDSSchriftKopf::addZeichen( unsigned char zeichen ) // Zeichen hinzufgen
{
	zeichenAnzahl++;
	unsigned char *zeichen_tmp = this->zeichen;
	int *pos_tmp = pos;
	this->zeichen = new unsigned char[ zeichenAnzahl ];
	pos = new int[ zeichenAnzahl ];
	if( zeichenAnzahl - 1 > 0 )
	{
		memcpy( this->zeichen, zeichen_tmp, 1 * ( zeichenAnzahl - 1 ) );
		memcpy( pos, pos_tmp, 4 * ( zeichenAnzahl - 1 ) );
	}
	delete []zeichen_tmp;
	delete []pos_tmp;
	this->zeichen[ zeichenAnzahl - 1 ] = zeichen;
	pos[ zeichenAnzahl - 1 ] = 0;
}

void LTDSSchriftKopf::removeZeichen( unsigned char zeich ) // Zeichen entfernen
{
	bool hatZ = 0;
	int zPos = 0;
	for( int i = 0; i < zeichenAnzahl; i++ )
	{
		hatZ = zeichen[ i ] == zeich;
		zPos = i;
		if( hatZ )
			break;
	}
	if( hatZ )
	{
		zeichenAnzahl--;
		unsigned char *zeichen_tmp = zeichen;
		int *pos_tmp = pos;
		zeichen = new unsigned char[ zeichenAnzahl ];
		pos = new int[ zeichenAnzahl ];
		for( int i = 0; i < zPos; i++ )
		{
			zeichen[ i ] = zeichen_tmp[ i ];
			pos[ i ] = pos_tmp[ i ];
		}
		for( int i = zPos + 1; i < zeichenAnzahl; i++ )
		{
			zeichen[ i - 1 ] = zeichen_tmp[ i ];
			pos[ i - 1 ] = pos_tmp[ i ];
		}
		delete []zeichen_tmp;
		delete []pos_tmp;
	}
}

// constant 
void LTDSSchriftKopf::speichern( std::ofstream *outF ) const // speichert nach outF
{
	if( outF->good() && outF->is_open() )
	{
		outF->write( (char*)&schriftGre, 1 );
		outF->write( (char*)&zeichenAnzahl, 1 );
		for( int i = 0; i < zeichenAnzahl; i++ )
		{
			outF->write( (char*)&zeichen[ i ], 1 );
			outF->write( (char*)&pos[ i ], 4 );
		}
	}
}

unsigned char LTDSSchriftKopf::getSchriftGre() const // gibt die Schriftgre zurck
{
	return schriftGre;
}

unsigned char LTDSSchriftKopf::getZeichenAnzahl() const // gibt die Zeichenanzahl zurck
{
	return zeichenAnzahl;
}

int *LTDSSchriftKopf::getPositionen() const // gibt die Zeichenpositionen zurck
{
	return pos;
}

unsigned char *LTDSSchriftKopf::getZeichen() const // gibt die zeichen zurck
{
	return zeichen;
}

// Reference Counting 
LTDSSchriftKopf *LTDSSchriftKopf::getThis()
{
	ref++;
	return this;
}

LTDSSchriftKopf *LTDSSchriftKopf::release()
{
	ref--;
	if( ref == 0 )
		delete this;
	return 0;
}

// Inhalt der LTDSBuchstabenKopf Klasse aus Dateisystem.h
// Konstruktor 
LTDSBuchstabenKopf::LTDSBuchstabenKopf()
{
	ref = 1;
	zeichen = 0;
	gre = new Punkt();
}

// Destruktor 
LTDSBuchstabenKopf::~LTDSBuchstabenKopf()
{
	gre->release();
}

// nicht constant 
void LTDSBuchstabenKopf::laden( std::ifstream *inF ) // lt aus inF
{
	if( inF->good() && inF->is_open() )
	{
		inF->read( (char*)&zeichen, 1 );
		inF->read( (char*)&gre->x, 1 );
		inF->read( (char*)&gre->y, 1 );
	}
}

void LTDSBuchstabenKopf::init( unsigned char zeichen, Punkt *gre ) // initialisierung( fr speichern )
{
	this->zeichen = zeichen;
	this->gre->setP( gre->x, gre->y );
	gre->release();
}

void LTDSBuchstabenKopf::init( unsigned char zeichen, int br, int h )
{
	this->zeichen = zeichen;
	gre->x = br, gre->y = h;
}

// constant 
void LTDSBuchstabenKopf::speichern( std::ofstream *outF ) const // speichertn nach outF
{
	if( outF->good() && outF->is_open() )
	{
		outF->write( (char*)&zeichen, 1 );
		outF->write( (char*)&gre->x, 1 );
		outF->write( (char*)&gre->y, 1 );
	}
}

unsigned char LTDSBuchstabenKopf::getZeichen() const // gibt das Zeichen zurck
{
	return zeichen;
}

int LTDSBuchstabenKopf::getBreite() const // gibt die Breite zurck
{
	return gre->x;
}

int LTDSBuchstabenKopf::getHhe() const // gibt die hhe zurck
{
	return gre->y;
}

Punkt *LTDSBuchstabenKopf::getGre() const // gibt die Gre zurck
{
	return gre->getThis();
}

// Reference Counting 
LTDSBuchstabenKopf *LTDSBuchstabenKopf::getThis()
{
	ref++;
	return this;
}

LTDSBuchstabenKopf *LTDSBuchstabenKopf::release()
{
	ref--;
	if( ref == 0 )
		delete this;
	return 0;
}

// Inhalt der LTDSBuchstabenKrper Klasse aus Dateisystem.h
// Konstruktor 
LTDSBuchstabenKrper::LTDSBuchstabenKrper( LTDSBuchstabenKopf *kopf )
{
	ref = 1;
	gre = new Punkt( kopf->getBreite(), kopf->getHhe() );
	zeichen = kopf->getZeichen();
	buchstabe = new Buchstabe();
	buchstabe->NeuBuchstabe( gre->getThis() );
	kopf->release();
}

// Destruktor 
LTDSBuchstabenKrper::~LTDSBuchstabenKrper()
{
	gre->release();
	if( buchstabe )
		buchstabe->release();
}

// nicht constant 
void LTDSBuchstabenKrper::setBuchstabe( Buchstabe *zeichen ) // setzt den Buchstaben
{
	if( buchstabe )
		buchstabe->release();
	buchstabe = zeichen;
}

void LTDSBuchstabenKrper::laden( std::ifstream *inF ) // Lht aus inF
{
	if( inF->good() && inF->is_open() )
	{
		LTDSPixel *vorher = 0;
		LTDSPixel *jetzt = new LTDSPixel( vorher );
		char byte = 0;
		int beg = 0;
		int ende = -1;
		for( int i = 0; i < gre->x * gre->y; i++ )
		{
			if( !jetzt ) // wenn es nicht der erste Pixel ist
				jetzt = new LTDSPixel( vorher->getThis() );
			int ende = -1;
			while( ende < 0 ) // Pixel laden
			{
				if( beg == 0 )
					inF->read( &byte, 1 );
				ende = jetzt->addByte( byte, beg ); // byte auswerten
				beg = 0;
			}
			beg = ende;
			if( beg == 8 )
				beg = 0;
			if( buchstabe )
				buchstabe->setPixel( i, jetzt->getA() );
			if( vorher )
				vorher = vorher->release();
			vorher = jetzt->getThis();
			jetzt = jetzt->release();
		}
		if( vorher )
			vorher->release();
	}
}

// constant 
void LTDSBuchstabenKrper::speichern( std::ofstream *outF ) const // speichert nach outF
{
	if( outF->good() && outF->is_open() )
	{
		LTDSPixel *vorher = 0; // Letzter gespeicherter Pixel
		LTDSPixel *jetzt = new LTDSPixel( 0 ); // Der momentan zu speichernde Pixel
		int begin = 0, ende = 0; // Pixelbeginn, endposition in der byte variable
		char byte = 0; // Der nchste byte der Datei
		bool w = 0;
		unsigned char *alphaBuff = buchstabe->getBuff();
		for( int i = 0; i < gre->getX() * gre->getY(); i++ ) // fr jeden Pixel
		{
			if( !jetzt ) // wenn es nicht der erste Pixel ist
				jetzt = new LTDSPixel( vorher->getThis() );
			jetzt->setAlpha( alphaBuff[ i ] ); // Farbe des Pixels setzen
			jetzt->Komp(); // Pixel komprimieren
			ende = -1;
			while( ende < 0 ) // byte befllen
			{
				ende = jetzt->getNextByte( byte, begin );
				begin = 0;
				w = 0;
				if( ende == -1 || ende == 8 ) // byte speichern
				{
					outF->write( &byte, 1 );
					w = 1;
					byte = 0;
				}
			} // Pixel fertig
			begin = ende;
			if( begin == 8 )
				begin = 0;
			if( vorher )
				vorher->release();
			vorher = jetzt->getThis(); // dieser wird zu letzter
			jetzt = jetzt->release();
		}
		if( vorher )
			vorher = vorher->release();
		if( !w )
		{
			outF->write( &byte, 1 ); // Das letzte byte speichern
		}
		outF->flush(); // dateistream speichern
	}
}

Buchstabe *LTDSBuchstabenKrper::getBuchstabe() const // gibt den Buchstaben zurck
{
	return buchstabe->getThis();
}

unsigned char LTDSBuchstabenKrper::getZeichen() const // gibt das Zeichen zurck
{
	return zeichen;
}

// Reference Counting 
LTDSBuchstabenKrper *LTDSBuchstabenKrper::getThis()
{
	ref++;
	return this;
}

LTDSBuchstabenKrper *LTDSBuchstabenKrper::release()
{
	ref--;
	if( ref == 0 )
		delete this;
	return 0;
}

// Inhalt der LTDSDatei Klasse aus DAteisystem.h
// Konstruktor 
LTDSDatei::LTDSDatei()
{
	ref = 1;
	pfad = new Text("");
	dateiKopf = 0;
	InitializeCriticalSection( &thread_Save );
}

// Destruktor 
LTDSDatei::~LTDSDatei()
{
	if( dateiKopf )
		dateiKopf->release();
	pfad->release();
	DeleteCriticalSection( &thread_Save );
}

// nicht constant 
void LTDSDatei::lockDatei()
{
	EnterCriticalSection( &thread_Save );
}

void LTDSDatei::unlockDatei()
{
	LeaveCriticalSection( &thread_Save );
}

void LTDSDatei::setPfad( Text *txt ) // setzt den Pfad zur Datei
{
	lockDatei();
	if( dateiKopf )
		dateiKopf = dateiKopf->release();
	pfad->setText( txt->getText() );
	txt->release();
	unlockDatei();
}

void LTDSDatei::leseDaten() // liet den Dateikopf
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) )
	{
		unlockDatei();
		return;
	}
	if( dateiKopf )
		dateiKopf->release();
	dateiKopf = new LTDSDateiKopf();
	std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary );
	dateiKopf->laden( inF );
	inF->close();
	unlockDatei();
	delete inF;
}

void LTDSDatei::addSchriftgre( Alphabet *alphabet ) // fgt eine Schriftgre hinzu
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) ) // prfen, ob die Datei existiert
	{
		unlockDatei();
		return;
	}
	if( !dateiKopf ) // prfen, ob der Dateikopf schon gelesen wurde
		leseDaten();
	int sgre = alphabet->getSchriftgre(); // Schriftgre die hinzugefgt werden soll
	unsigned char *sglist = dateiKopf->getSchriftGreList(); // Liste von bereits vorhandenen Schriftgren
	int *sgPosList = dateiKopf->getPositionList(); // Liste mit positionen der Schriftgren
	unsigned char sganzahl = dateiKopf->getSchriftGreAnzahl(); // Anzahl der bereits vorhandenen Schriftgren
	for( int i = 0; i < sganzahl; i++ ) // prfen, ob die Schriftgre bereits existiert
	{
		if( sglist[ i ] == sgre )
		{
			unlockDatei();
			return;
		}
	}
	dateiKopf->addSG( sgre ); // Schriftgre dem Dateikopf hinzufgen
	std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary ); // alte Datei
	pfad->anhngen( "0" );
	std::ofstream *outF = new std::ofstream( pfad->getText(), std::ios::binary ); // neue Datei
	dateiKopf->speichern( outF ); // Dateikopf in neue datei speichern
	inF->seekg( 1 + 5 * sganzahl, std::ios::beg ); // Position der ersten Schriftgre in der alten Datei
	for( int i = 0; i < sganzahl; i++ ) // Buchstabenpositionen aller Schriftgren aktualisieren
	{
		LTDSSchriftKopf *sgKpf_tmp = new LTDSSchriftKopf(); // Schriftgren Kopf
		sgKpf_tmp->laden( inF ); // aus alter Datei laden
		int *zeichP_tmp = sgKpf_tmp->getPositionen(); // Zeichenpositionen der Schriftgre
		unsigned char zeichA_tmp = sgKpf_tmp->getZeichenAnzahl(); // Anzahl von Zeichen der Schriftgre
		for( int i1 = 0; i1 < zeichA_tmp; i1++ )
			zeichP_tmp[ i1 ] += 5; // Buchstabenpositionen aktualisieren
		sgKpf_tmp->speichern( outF ); // Schriftgre Kopf in neue Datei speichern
		int beginByte = (int)inF->tellg(); // Die restlichen bytes bis zur nchsten Schriftgre in neue Datei kopieren
		int endByte = sgPosList[ i + 1 ];
		if( !endByte )
		{
			inF->seekg( 0, std::ios::end );
			endByte = (int)inF->tellg();
			inF->seekg( beginByte, std::ios::beg );
		}
		char byte;
		for( int i1 = beginByte; i1 < endByte; i1++ ) // Kopiervorgang
		{
			inF->read( &byte, 1 );
			outF->write( &byte, 1 );
		}
		sgKpf_tmp->release();
	}
	inF->close(); // Alte datei schlieen
	sgPosList[ sganzahl ] = (int)outF->tellp();
	outF->seekp( 0, std::ios::beg );
	for( int i = 0; i < sganzahl; i++ ) // Positionen im Dateikopf aktualisieren
		sgPosList[ i ] += 5;
	dateiKopf->speichern( outF ); // aktualisierter Dateikopf speichern
	outF->seekp( sgPosList[ sganzahl ], std::ios::beg );
	LTDSSchriftKopf *sgkopf = new LTDSSchriftKopf(); // Kopf der neuen Schriftgre
	sgkopf->setZeichenAlphabet( alphabet->getThis() ); // Kopf der Schriftgre initialisieren
	sgkopf->speichern( outF ); // Kopf der Schriftgre speichern
	int *BuchstabenPosList = sgkopf->getPositionen(); // positionen der verschiedenen Zeichen in der Datei( nuch 0 )
	int count = 0;
	for( int i = 0; i < 256; i++ )
	{
		Buchstabe *zeich = alphabet->getBuchstabe( i );
		if( zeich )
		{
			BuchstabenPosList[ count ] = (int)outF->tellp(); // position des Zeichens setzen
			LTDSBuchstabenKopf *zeichKpf = new LTDSBuchstabenKopf(); // Zeichenkopf
			zeichKpf->init( i, zeich->getBreite(), zeich->getHhe() );
			zeichKpf->speichern( outF ); // Zeichenkopf speichern
			LTDSBuchstabenKrper *zeichKrp = new LTDSBuchstabenKrper( zeichKpf->getThis() ); // Zeichenkrper
			zeichKpf->release();
			zeichKrp->setBuchstabe( zeich->getThis() );
			zeichKrp->speichern( outF ); // Zeichenkrper speichern
			zeich->release();
			count++;
		}
	}
	outF->seekp( sgPosList[ sganzahl ], std::ios::beg );
	sgkopf->speichern( outF ); // aktualisierter Schriftgren Kopf speichern
	outF->close();
	Text *pfad2 = new Text();
	pfad2->setText( pfad->getText() );
	pfad->lschen( pfad->getLnge() - 1, pfad->getLnge() );
	DateiLschen( pfad->getThis() ); // Alte datei Lschen
	DateiUmbenennen( pfad2->getThis(), pfad->getThis() ); // neue Datei nach alte umbenennen
	pfad->release(); // Speicher freigeben
	sgkopf->release();
	delete inF;
	delete outF;
	alphabet->release();
	unlockDatei();
}

void LTDSDatei::addBuchstabe( int gr, Buchstabe *zeich, unsigned char zeichen ) // Fgt einer Schriftgre einen Buchstaben hinzu
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) ) // prfen ob Datei existiert
	{
		unlockDatei();
		return;
	}
	if( !dateiKopf ) // prfen, ob der DAteikopf geladen wurde
		leseDaten();
	unsigned char *sgList = dateiKopf->getSchriftGreList(); // Liste mit Schriftgren aus der Datei
	int *sgPosList = dateiKopf->getPositionList(); // Liste mit Schriftgren positionen aus Datei
	unsigned char sgAnzahl = dateiKopf->getSchriftGreAnzahl(); // Anzahl von Schriftgren aus der Datei
	int sgNum = -1;
	for( int i = 0; i < sgAnzahl; i++ ) // Position der richtigen Schriftgre ermitteln
	{
		if( sgList[ i ] == gr )
		{
			sgNum = i;
			break;
		}
	}
	std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary ); // Alte Datei
	pfad->anhngen( "0" );
	std::ofstream *outF = new std::ofstream( pfad->getText(), std::ios::binary ); // Neue Datei
	inF->seekg( 1 + 5 * sgAnzahl, std::ios::beg );
	if( sgNum == -1 ) // Die Schriftgre existiert noch nicht und wird erstellt
	{
		dateiKopf->addSG( gr ); // Schriftgre dem Dateikopf hinzufgen
		sgPosList = dateiKopf->getPositionList();
		sgList = dateiKopf->getSchriftGreList();
		dateiKopf->speichern( outF ); // Dateikopf speichern
		inF->seekg( 1 + 5 * sgAnzahl, std::ios::beg ); // Position der ersten Schriftgre in der alten Datei
		for( int i = 0; i < sgAnzahl; i++ ) // Buchstabenpositionen aller Schriftgren aktualisieren
		{
			LTDSSchriftKopf *sgKpf_tmp = new LTDSSchriftKopf(); // Schriftgren Kopf
			sgKpf_tmp->laden( inF ); // aus alter Datei laden
			int *zeichP_tmp = sgKpf_tmp->getPositionen(); // Zeichenpositionen der Schriftgre
			unsigned char zeichA_tmp = sgKpf_tmp->getZeichenAnzahl(); // Anzahl von Zeichen der Schriftgre
			for( int i1 = 0; i1 < zeichA_tmp; i1++ )
				zeichP_tmp[ i1 ] += 5; // Buchstabenpositionen aktualisieren
			sgKpf_tmp->speichern( outF ); // Schriftgre Kopf in neue Datei speichern
			int beginByte = (int)inF->tellg(); // Die restlichen bytes bis zur nchsten Schriftgre in neue Datei kopieren
			int endByte = sgPosList[ i + 1 ];
			if( i + 1 >= sgAnzahl )
			{
				inF->seekg( 0, std::ios::end );
				endByte = (int)inF->tellg();
				inF->seekg( beginByte, std::ios::beg );
			}
			char byte;
			for( int i1 = beginByte; i1 < endByte; i1++ ) // Kopiervorgang
			{
				inF->read( &byte, 1 );
				outF->write( &byte, 1 );
			}
			sgKpf_tmp->release();
		}
		sgPosList[ sgAnzahl ] = (int)outF->tellp();
		outF->seekp( 0, std::ios::beg );
		for( int i = 0; i < sgAnzahl; i++ ) // Positionen im Dateikopf aktualisieren
			sgPosList[ i ] += 5;
		dateiKopf->speichern( outF ); // aktualisierter Dateikopf speichern
		outF->seekp( sgPosList[ sgAnzahl ], std::ios::beg );
		LTDSSchriftKopf *sgKpf = new LTDSSchriftKopf(); // Schriftgre Kopf initialisiern
		sgKpf->setSchriftgre( gr );
		sgKpf->addZeichen( zeichen );
		sgKpf->getPositionen()[ 0 ] = (int)outF->tellp() + 7;
		sgKpf->speichern( outF ); // Schriftgre Kopf speichern
		sgKpf->release();
		LTDSBuchstabenKopf *zeichKpf = new LTDSBuchstabenKopf(); // Buchstabenkopf
		zeichKpf->init( zeichen, zeich->getBreite(), zeich->getHhe() );
		zeichKpf->speichern( outF ); // Buchstabenkopf speichern
		LTDSBuchstabenKrper *zeichKrp = new LTDSBuchstabenKrper( zeichKpf->getThis() ); // Buchstabenkrper
		zeichKpf->release();
		zeichKrp->setBuchstabe( zeich->getThis() );
		zeichKrp->speichern( outF ); // Buchstabenkrper speichern
		zeichKrp->release();
	}
	else
	{
		dateiKopf->speichern( outF ); // Dateikopf speichern
		int beginByte = 1 + 5 * sgAnzahl; // Die restlichen bytes bis zur nchsten Schriftgre in neue Datei kopieren
		int endByte = sgPosList[ sgNum ];
		char byte;
		for( int i1 = beginByte; i1 < endByte; i1++ ) // Kopiervorgang bis zur angegebenen gre
		{
			inF->read( &byte, 1 );
			outF->write( &byte, 1 );
		}
		LTDSSchriftKopf *sgKpf = new LTDSSchriftKopf(); // Schriftgre Kopf
		sgKpf->laden( inF );
		for( int i = 0; i < sgKpf->getZeichenAnzahl(); i++ )
			sgKpf->getPositionen()[ i ] += 5;
		sgKpf->addZeichen( zeichen );
		int indexPlus = 5;
		int zeichenPos = sgPosList[ sgNum + 1 ]; // positione des neuen Zeichens
		if( sgNum + 1 >= sgAnzahl )
		{
			int tmp = (int)inF->tellg();
			inF->seekg( 0, std::ios::end );
			zeichenPos = (int)inF->tellg();
			inF->seekg( tmp, std::ios::beg );
		}
		zeichenPos += indexPlus;
		sgKpf->getPositionen()[ sgKpf->getZeichenAnzahl() - 1 ] = zeichenPos;
		sgKpf->speichern( outF ); // Schriftgre Kopf speichern
		sgKpf->release();
		for( int i = (int)inF->tellg() + indexPlus; i < zeichenPos; i++ ) // Kopiervorgang bis zum Zeichenbeginn
		{
			inF->read( &byte, 1 );
			outF->write( &byte, 1 );
		}
		LTDSBuchstabenKopf *zeichKpf = new LTDSBuchstabenKopf(); // Zeichenkopf
		zeichKpf->init( zeichen, zeich->getBreite(), zeich->getHhe() );
		zeichKpf->speichern( outF ); // Zeichenkopf speichern
		LTDSBuchstabenKrper *zeichKrp = new LTDSBuchstabenKrper( zeichKpf->getThis() ); // Zeichenkrper
		zeichKpf->release();
		zeichKrp->setBuchstabe( zeich->getThis() );
		zeichKrp->speichern( outF ); // Zeichenkrper speichern
		zeichKrp->release();
		int nowPos = (int)outF->tellp();
		indexPlus += nowPos - zeichenPos;
		for( int i = sgNum + 1; i < sgAnzahl; i++ ) // Dateikopf aktualisieren
			sgPosList[ i ] += indexPlus;
		outF->seekp( 0, std::ios::beg );
		dateiKopf->speichern( outF ); // Dateikopf speichern
		outF->seekp( nowPos, std::ios::beg );
		for( int i = sgNum + 1; i < sgAnzahl; i++ ) // Restliche Schriftgren aktualisieren
		{
			LTDSSchriftKopf *sgKpf_tmp = new LTDSSchriftKopf(); // Schriftgren Kopf
			sgKpf_tmp->laden( inF ); // aus alter Datei laden
			int *zeichP_tmp = sgKpf_tmp->getPositionen(); // Zeichenpositionen der Schriftgre
			unsigned char zeichA_tmp = sgKpf_tmp->getZeichenAnzahl(); // Anzahl von Zeichen der Schriftgre
			for( int i1 = 0; i1 < zeichA_tmp; i1++ )
				zeichP_tmp[ i1 ] += indexPlus; // Buchstabenpositionen aktualisieren
			sgKpf_tmp->speichern( outF ); // Schriftgre Kopf in neue Datei speichern
			int beginByte = (int)inF->tellg(); // Die restlichen bytes bis zur nchsten Schriftgre in neue Datei kopieren
			int endByte = sgPosList[ i + 1 ];
			if( i + 1 >= sgAnzahl )
			{
				inF->seekg( 0, std::ios::end );
				endByte = (int)inF->tellg();
				inF->seekg( beginByte, std::ios::beg );
			}
			char byte;
			for( int i1 = beginByte; i1 < endByte; i1++ ) // Kopiervorgang
			{
				inF->read( &byte, 1 );
				outF->write( &byte, 1 );
			}
			sgKpf_tmp->release();
		}
	}
	inF->close();
	outF->close();
	Text *pfad2 = new Text( pfad->getText() );
	pfad->lschen( pfad->getLnge() - 1, pfad->getLnge() );
	DateiLschen( pfad->getThis() ); // Alte Datei lschen
	DateiUmbenennen( pfad2->getThis(), pfad->getThis() ); // Neue Datei in alte umbenennen
	pfad2->release();// Speicher freigeben
	delete inF;
	delete outF;
	zeich->release();
	unlockDatei();
}

void LTDSDatei::lscheSchrifrGre( int gr ) // Lscht eine Schriftgre aus der Datei
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) ) // prfen, ob Datei existiert
	{
		unlockDatei();
		return;
	}
	if( !dateiKopf ) // prfen, ob der Dateikopf geladen wurde
		leseDaten();
	unsigned char *sgList = dateiKopf->getSchriftGreList(); // Liste mit Schriftgren
	unsigned char sgAnzahl = dateiKopf->getSchriftGreAnzahl(); // Anzahl der Schriftgren
	int sgNum = -1;
	for( int i = 0; i < sgAnzahl; i++ ) // zu lschende Schriftgre suchen
	{
		if( sgList[ i ] == gr )
		{
			sgNum = i;
			break;
		}
	}
	if( sgNum == -1 ) // Die Schriftgre ist nicht vorhanden
	{
		unlockDatei();
		return;
	}
	int *sgPosList = dateiKopf->getPositionList();  // Liste von Positionen der Schriftgren
	std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary ); // alte Datei
	pfad->anhngen( "0" );
	std::ofstream *outF = new std::ofstream( pfad->getText(), std::ios::binary ); // neue Datei
	outF->seekp( 1 + 5 * (sgAnzahl-1), std::ios::beg );
	inF->seekg( 1 + 5 * sgAnzahl, std::ios::beg );
	for( int i = 0; i < sgNum; i++ ) // Schriftgren vor der zu lschenden Schriftgre aktualisieren
	{
		LTDSSchriftKopf *sgKpf_tmp = new LTDSSchriftKopf(); // Schriftgren Kopf
		sgKpf_tmp->laden( inF ); // Schriftgren Kopf laden
		int *zeichPosLTmp = sgKpf_tmp->getPositionen(); // Zeichenpositionen
		unsigned char zeichATmp = sgKpf_tmp->getZeichenAnzahl(); // Zeichenanzahl
		for( int i1 = 0; i1 < zeichATmp; i1++ ) // Position der Zeichen um 5 bytes zurcksetzen
			zeichPosLTmp[ i1 ] -= 5;
		sgKpf_tmp->speichern( outF ); // speichern in Neue Datei
		char byte = 0;
		for( int i1 = (int)inF->tellg(); i1 < sgPosList[ i+1 ]; i1++ ) // Den Krper des Zeichens Kopieren
		{
			inF->read( &byte, 1 );
			outF->write( &byte, 1 );
		}
	}
	int indexMinus = 5 + sgPosList[ sgNum + 1 ] - (int) inF->tellg();
	inF->seekg( sgPosList[ sgNum + 1 ], std::ios::beg );
	for( int i = sgNum + 1; i < sgAnzahl; i++ ) // Die Schriftgre nach der zu lschenden Schriftgre
	{
		LTDSSchriftKopf *sgKpf_tmp = new LTDSSchriftKopf(); // Schriftgren Kopf
		sgKpf_tmp->laden( inF ); // Schriftgren Kopf laden
		int *zeichPosLTmp = sgKpf_tmp->getPositionen(); // Zeichenpositionen
		unsigned char zeichATmp = sgKpf_tmp->getZeichenAnzahl(); // Zeichenanzahl
		for( int i1 = 0; i1 < zeichATmp; i1++ ) // Position der Zeichen aktualisieren
			zeichPosLTmp[ i1 ] -= indexMinus;
		sgKpf_tmp->speichern( outF ); // speichern in Neue Datei
		char byte = 0;
		int BeginByte = (int)inF->tellg();
		int EndByte = sgPosList[ i+1 ];
		if( !EndByte )
		{
			inF->seekg( 0, std::ios::end );
			EndByte = (int)inF->tellg();
			inF->seekg( BeginByte, std::ios::beg );
		}
		for( int i1 = BeginByte; i1 < EndByte; i1++ ) // Den Krper des Zeichens Kopieren
		{
			inF->read( &byte, 1 );
			outF->write( &byte, 1 );
		}
	}
	for( int i = 0; i < sgNum; i++ ) // Dateikopf aktualisieren
		sgPosList[ i ] -= 5;
	for( int i = sgNum + 1; i < sgAnzahl; i++ )
		sgPosList[ i ] -= indexMinus;
	dateiKopf->removeSG( gr );
	outF->seekp( 0, std::ios::beg );
	dateiKopf->speichern( outF ); // Dateikopf speichern
	inF->close();
	outF->close();
	Text *pfad2 = new Text( pfad->getText() );
	pfad->lschen( pfad->getLnge() - 1, pfad->getLnge() );
	DateiLschen( pfad->getThis() ); // alte Datei lschen
	DateiUmbenennen( pfad2->getThis(), pfad->getThis() ); // neue Datei zu alter umbenennen
	pfad2->release();
	delete inF;
	delete outF;
	unlockDatei();
}

void LTDSDatei::lscheBuchstabe( int gr, unsigned char zeichen ) // Lscht einen Buchstaben aus der Datei
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) ) // prfen, ob die Datei existiert
	{
		unlockDatei();
		return;
	}
	if( !dateiKopf ) // prfen, ob der Dateikopf gelesen wurde
		leseDaten();
	unsigned char *sgList = dateiKopf->getSchriftGreList(); // Liste mit Schriftgren
	unsigned char sgAnzahl = dateiKopf->getSchriftGreAnzahl(); // Anzahl der Schriftgren
	int *sgPosList = dateiKopf->getPositionList(); // Liste mit Positionen der Schriftgren
	int sgNum = -1;
	for( int i = 0; i < sgAnzahl; i++ ) // Schriftgre suchen
	{
		if( sgList[ i ] == gr )
		{
			sgNum = i;
			break;
		}
	}
	if( sgNum == -1 ) // Schriftgre nicht gefunden
	{
		unlockDatei();
		return;
	}
	std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary ); // alte Datei
	pfad->anhngen( "0" );
	std::ofstream *outF = new std::ofstream( pfad->getText(), std::ios::binary ); // neue Datei
	int indexMinus = 0;
	inF->seekg( 1 + 5 * sgAnzahl, std::ios::beg );
	dateiKopf->speichern( outF ); // DateiKopf peichern
	for( int i = 0; i < sgAnzahl; i++ )
	{
		LTDSSchriftKopf *sgKopf = new LTDSSchriftKopf(); // Schriftkopf
		sgKopf->laden( inF ); // Schriftkopf laden
		unsigned char sgZeichAnzahl = sgKopf->getZeichenAnzahl(); // Zeichenanzahl
		unsigned char *sgZeichenList = sgKopf->getZeichen(); // Zeichen Liste
		int *sgZPosList = sgKopf->getPositionen(); // Positions Liste
		if( i == sgNum ) // Zeichen befindet sich in dieser Schriftgre
		{
			int sgZNum = -1;
			for( int i1 = 0; i1 < sgZeichAnzahl; i1++ ) // Zeichen suchen
			{
				if( sgZeichenList[ i1 ] == zeichen )
				{
					sgZNum = i1;
					break;
				}
			}
			if( sgZNum == -1 ) // Zeichen nicht gefunden
			{
				sgKopf->release();
				inF->close();
				outF->close();
				delete inF;
				delete outF;
				DateiLschen( pfad->getThis() );
				pfad->lschen( pfad->getLnge() - 1, pfad->getLnge() );
				unlockDatei();
				return; // abbruch
			}
			outF->seekp( 2 + 5 * (sgZeichAnzahl-1), std::ios::cur );
			indexMinus += 5;
			for( int i1 = 0; i1 < sgZNum; i1++ ) // Zeichen vor dem zu lschenden Zeichen
			{
				char byte = 0;
				for( int i2 = sgZPosList[ i1 ]; i2 < sgZPosList[ i1 + 1 ]; i2++ ) // Kopieren
				{
					inF->read( &byte, 1 );
					outF->write( &byte, 1 );
				}
				sgZPosList[ i1 ] -= indexMinus; // Schriftgre Kopf aktualisieren
			}
			indexMinus += sgZPosList[ sgZNum + 1 ] - sgZPosList[ sgZNum ];
			if( sgZNum + 1 < sgZeichAnzahl )
				inF->seekg( sgZeichenList[ sgZNum + 1 ], std::ios::beg );
			for( int i1 = sgZNum + 1; i1 < sgZeichAnzahl; i1++ ) // Zeichen nach dem gelschten Zeichen
			{
				int beginByte = (int)inF->tellg();
				int endByte = sgZPosList[ i1 + 1 ];
				if( !endByte )
				{
					inF->seekg( 0, std::ios::end );
					endByte = (int)inF->tellg();
					inF->seekg( beginByte, std::ios::beg );
				}
				char byte = 0;
				for( int i2 = beginByte; i2 < endByte; i2++ ) // Kopieren
				{
					inF->read( &byte, 1 );
					outF->write( &byte, 1 );
				}
				sgZPosList[ i1 ] -= indexMinus; // Schriftgre Kopf aktualisieren
			}
			sgKopf->removeZeichen( zeichen );
		}
		else
		{
			for( int i1 = 0; i1 < sgZeichAnzahl; i1++ ) // Schriftgre Kopf aktualisieren
				sgZPosList[ i ] -= indexMinus;
			sgKopf->speichern( outF ); // Schriftgre Kopf speichern
			int beginByte = (int)inF->tellg();
			int endByte = sgPosList[ i + 1 ];
			if( !endByte )
			{
				inF->seekg( 0, std::ios::end );
				endByte = (int)inF->tellg();
				inF->seekg( beginByte, std::ios::beg );
			}
			char byte;
			for( int i1 = beginByte; i1 < endByte; i1++ ) // Kopieren
			{
				inF->read( &byte, 1 );
				outF->write( &byte, 1 );
			}
			sgPosList[ i ] -= indexMinus;
		}
		outF->seekp( sgPosList[ i ], std::ios::beg );
		sgKopf->speichern( outF ); // Schriftgre Kopf speichern
		outF->seekp( sgPosList[ i + 1 ], std::ios::beg );
		sgKopf->release();
	}
	inF->close();
	outF->close();
	Text *pfad2 = new Text( pfad->getText() );
	pfad->lschen( pfad->getLnge() - 1, pfad->getLnge() );
	DateiLschen( pfad->getThis() ); // alte Datei lschen
	DateiUmbenennen( pfad2->getThis(), pfad->getThis() ); // neue Datei nach alte umbenennen
	pfad2->release(); // Speicher freigeben
	delete inF;
	delete outF;
	unlockDatei();
}

void LTDSDatei::lscheDatei() // Lscht die gesamte Datei
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) )
	{
		unlockDatei();
		return;
	}
	if( !dateiKopf )
		leseDaten();
	DateiLschen( pfad->getThis() );
	unlockDatei();
}

void LTDSDatei::erstelleDatei() // erstellt die Datei
{
	lockDatei();
	DateiPfadErstellen( pfad->getThis() );
	if( dateiKopf )
		dateiKopf->release();
	dateiKopf = new LTDSDateiKopf();
	std::ofstream *outF = new std::ofstream( pfad->getText(), std::ios::binary );
	dateiKopf->speichern( outF );
	outF->close();
	unlockDatei();
	delete outF;
}

void LTDSDatei::speicherSchrift( Schrift *schrift ) // Speichert die bergebene Schrift
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) )
	{
		unlockDatei();
		return;
	}
	lscheDatei();
	if( dateiKopf )
		dateiKopf->release();
	dateiKopf = new LTDSDateiKopf();
	for( int i = 0; i < schrift->getAlphabetAnzahl(); i++ )
	{
		Alphabet *alp = schrift->getAlphabet( i );
		dateiKopf->addSG( alp->getSchriftgre() );
		alp->release();
	}
	std::ofstream *outF = new std::ofstream( pfad->getText(), std::ios::binary );
	dateiKopf->speichern( outF );
	for( int i = 0; i < schrift->getAlphabetAnzahl(); i++ )
	{
		dateiKopf->getPositionList()[ i ] = (int)outF->tellp();
		LTDSSchriftKopf *sgKpf = new LTDSSchriftKopf();
		sgKpf->setZeichenAlphabet( schrift->getAlphabetI( i ) );
		sgKpf->speichern( outF );
		Alphabet *alp = schrift->getAlphabetI( i );
		for( int i1 = 0; i1 < sgKpf->getZeichenAnzahl(); i++ )
		{
			sgKpf->getPositionen()[ i1 ] = (int)outF->tellp();
			LTDSBuchstabenKopf *zeichKpf = new LTDSBuchstabenKopf();
			Buchstabe *zeichen = alp->getBuchstabe( sgKpf->getZeichen()[ i1 ] );
			zeichKpf->init( sgKpf->getZeichen()[ i1 ], zeichen->getBreite(), zeichen->getHhe() );
			zeichKpf->speichern( outF );
			LTDSBuchstabenKrper *zeichKrp = new LTDSBuchstabenKrper( zeichKpf->getThis() );
			zeichKrp->setBuchstabe( zeichen->getThis() );
			zeichKrp->speichern( outF );
			zeichKrp->release();
			zeichen->release();
			zeichKpf->release();
		}
		alp->release();
		outF->seekp( dateiKopf->getPositionList()[ i ], std::ios::beg );
		sgKpf->speichern( outF );
		outF->seekp( 0, std::ios::beg );
		sgKpf->release();
	}
	outF->seekp( 0, std::ios::beg );
	dateiKopf->speichern( outF );
	outF->close();
	delete outF;
	unlockDatei();
}

// constant 
Schrift *LTDSDatei::ladeSchrift() // gibt die geladene Schrift zurck
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) )
	{
		unlockDatei();
		return 0;
	}
	if( !dateiKopf )
	{
		unlockDatei();
		return 0;
	}
	Schrift *ret = new Schrift();
	std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary );
	inF->seekg( dateiKopf->getPositionList()[ 0 ], std::ios::beg );
	for( int i = 0; i < dateiKopf->getSchriftGreAnzahl(); i++ )
	{
		LTDSSchriftKopf *sgKpf = new LTDSSchriftKopf();
		sgKpf->laden( inF );
		Alphabet *alphabet = new Alphabet();
		alphabet->setSchriftgre( sgKpf->getSchriftGre() );
		alphabet->setDrawSchriftgre( sgKpf->getSchriftGre() );
		for( int i1 = 0; i1 < sgKpf->getZeichenAnzahl(); i1++ )
		{
			LTDSBuchstabenKopf *zeichKpf = new LTDSBuchstabenKopf();
			zeichKpf->laden( inF );
			LTDSBuchstabenKrper *zeichKrp = new LTDSBuchstabenKrper( zeichKpf->getThis() );
			zeichKrp->laden( inF );
			alphabet->setBuchstabe( zeichKpf->getZeichen(), zeichKrp->getBuchstabe() );
			zeichKrp->release();
			zeichKpf->release();
		}
		ret->addAlphabet( alphabet->getThis() );
		alphabet->release();
		sgKpf->release();
	}
	inF->close();
	delete inF;
	return ret;
	unlockDatei();
}

Alphabet *LTDSDatei::ladeAlphabet( int schriftgre ) // gibt eine geladene Schrift nur mit der angegebenen Schriftgre zurck
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) )
	{
		unlockDatei();
		return 0;
	}
	if( !dateiKopf )
	{
		unlockDatei();
		return 0;
	}
	Alphabet *ret = 0;
	int sgNum = -1;
	for( int i = 0; i < dateiKopf->getSchriftGreAnzahl(); i++ )
	{
		if( dateiKopf->getSchriftGreList()[ i ] == schriftgre )
		{
			sgNum = i;
			break;
		}
	}
	if( sgNum == -1 )
	{
		unlockDatei();
		return 0;
	}
	ret = new Alphabet();
	ret->NeuAlphabet();
	ret->setSchriftgre( schriftgre );
	ret->setDrawSchriftgre( schriftgre );
	std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary );
	inF->seekg( dateiKopf->getPositionList()[ sgNum ], std::ios::beg );
	LTDSSchriftKopf *sgKpf = new LTDSSchriftKopf();
	sgKpf->laden( inF );
	for( int i = 0; i < sgKpf->getZeichenAnzahl(); i++ )
	{
		LTDSBuchstabenKopf *sgZKpf = new LTDSBuchstabenKopf();
		sgZKpf->laden( inF );
		LTDSBuchstabenKrper *sgZKrp = new LTDSBuchstabenKrper( sgZKpf->getThis() );
		sgZKrp->laden( inF );
		ret->setBuchstabe( sgZKpf->getZeichen(), sgZKrp->getBuchstabe() );
		sgZKrp->release();
		sgZKpf->release();
	}
	sgKpf->release();
	inF->close();
	delete inF;
	unlockDatei();
	return ret;
}

Buchstabe *LTDSDatei::ladeBuchstabe( int schriftgre, unsigned char zeichen )// Ld einen bestimmten Buchstaben
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) )
	{
		unlockDatei();
		return 0;
	}
	if( !dateiKopf )
	{
		unlockDatei();
		return 0;
	}
	Buchstabe *ret = 0;
	int sgNum = -1;
	for( int i = 0; i < dateiKopf->getSchriftGreAnzahl(); i++ )
	{
		if( dateiKopf->getSchriftGreList()[ i ] == schriftgre )
		{
			sgNum = i;
			break;
		}
	}
	if( sgNum == -1 )
	{
		unlockDatei();
		return 0;
	}
	std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary );
	inF->seekg( dateiKopf->getPositionList()[ sgNum ], std::ios::beg );
	LTDSSchriftKopf *sgKpf = new LTDSSchriftKopf();
	sgKpf->laden( inF );
	int sgZNum = -1;
	for( int i = 0; i < sgKpf->getZeichenAnzahl(); i++ )
	{
		if( sgKpf->getZeichen()[ i ] == zeichen )
		{
			sgZNum = i;
			break;
		}
	}
	if( sgZNum != -1 )
	{
		inF->seekg( sgKpf->getPositionen()[ sgZNum ], std::ios::beg );
		LTDSBuchstabenKopf *sgZKpf = new LTDSBuchstabenKopf();
		sgZKpf->laden( inF );
		LTDSBuchstabenKrper *sgZKrp = new LTDSBuchstabenKrper( sgZKpf->getThis() );
		sgZKrp->laden( inF );
		ret = sgZKrp->getBuchstabe();
		sgZKrp->release();
		sgZKpf->release();
	}
	sgKpf->release();
	inF->close();
	delete inF;
	unlockDatei();
	return ret;
}

Text *LTDSDatei::getPfad() const // gibt den Dateipfad zurck
{
	return pfad->getThis();
}

int LTDSDatei::getAnzahlSchriftgren() const // gibt die Anzahl der Schriftgren aus der Datei zurck
{
	if( !DateiExistiert( pfad->getThis() ) )
		return 0;
	if( !dateiKopf )
		return 0;
	return dateiKopf->getSchriftGreAnzahl();
}

unsigned char *LTDSDatei::getSchriftGren() const // gibt einen Array von Schriftgren zurck
{
	if( !DateiExistiert( pfad->getThis() ) )
		return 0;
	if( !dateiKopf )
		return 0;
	return dateiKopf->getSchriftGreList();
}

unsigned char LTDSDatei::getAnzahlBuchstaben( int sg ) // gibt die anzahl gespeicherter Buchstaben einer Schriftgre zurck
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) )
	{
		unlockDatei();
		return 0;
	}
	if( !dateiKopf )
	{
		unlockDatei();
		return 0;
	}
	int ret = 0;
	unsigned char *gren = dateiKopf->getSchriftGreList();
	unsigned char granzahl = dateiKopf->getSchriftGreAnzahl();
	int grpos = -1;
	for( int i = 0; i < granzahl; i++ )
	{
		if( gren[ i ] == sg )
		{
			grpos = i;
			break;
		}
	}
	if( grpos != -1 )
	{
		int *grposlist = dateiKopf->getPositionList();
		LTDSSchriftKopf *sgkpf = new LTDSSchriftKopf();
		std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary );
		inF->seekg( grposlist[ grpos ], std::ios::beg );
		sgkpf->laden( inF );
		ret = sgkpf->getZeichenAnzahl();
		sgkpf->release();
		inF->close();
		delete inF;
	}
	unlockDatei();
	return ret;
}

unsigned char *LTDSDatei::getBuchstaben( int sg ) // gibt einen Array von Buchstaben einer Schriftgre zurck
{
	lockDatei();
	if( !DateiExistiert( pfad->getThis() ) )
	{
		unlockDatei();
		return 0;
	}
	if( !dateiKopf )
	{
		unlockDatei();
		return 0;
	}
	unsigned char *ret = 0;
	unsigned char *gren = dateiKopf->getSchriftGreList();
	unsigned char granzahl = dateiKopf->getSchriftGreAnzahl();
	int grpos = -1;
	for( int i = 0; i < granzahl; i++ )
	{
		if( gren[ i ] == sg )
		{
			grpos = i;
			break;
		}
	}
	if( grpos != -1 )
	{
		int *grposlist = dateiKopf->getPositionList();
		LTDSSchriftKopf *sgkpf = new LTDSSchriftKopf();
		std::ifstream *inF = new std::ifstream( pfad->getText(), std::ios::binary );
		inF->seekg( grposlist[ grpos ], std::ios::beg );
		sgkpf->laden( inF );
		ret = new unsigned char[ sgkpf->getZeichenAnzahl() ];
		for( int i = 0; i < sgkpf->getZeichenAnzahl(); i++ )
			ret[ i ] = sgkpf->getZeichen()[ i ];
		sgkpf->release();
		inF->close();
		delete inF;
	}
	unlockDatei();
	return ret;
}

// Reference Counting 
LTDSDatei *LTDSDatei::getThis()
{
	ref++;
	return this;
}

LTDSDatei *LTDSDatei::release()
{
	ref--;
	if( ref == 0 )
		delete this;
	return 0;
}

// Bit Funktionen
int Framework::Bits( int a ) // gibt 1-bits in gewinschter anzahl zurck
{
	int ret = 0;
	for( int i = 0; i < a; i++ )
	{
		ret <<= 1;
		ret += 1;
	}
	return ret;
}

int Framework::getBits( char c ) // gibt zurck, wie viele Bits c bentigt
{
	int ret = 0;
	for( int i = 0; ( c & ( char )Bits( i ) ) != c ; i++ )
	{
		ret++;
	}
	return ret;
}