#ifndef DateiSystem_H
#define DateiSystem_H

#include <fstream>
#include "Array.h"

namespace Framework
{
	class Bild; // Bild.h
	class Text; // Text.h
	class Punkt; // Punkt.h
	class Farbe; // Farbe.h
	class Buchstabe; // Schrift.h
	class Alphabet; // Schrift.h
	class Schrift; // Schrift.h
	class FBalken; // Fortschritt.h
	class LTDSPixel; // aus dieser Datei
	class LTDSDateiKopf; // aus dieser Datei
	class LTDSSchriftKopf; // aus dieser Datei
	class LTDSBuchstabenKopf; // aus dieser Datei
	class LTDSBuchstabenKrper; // aus dieser Datei
	class LTDSDatei; // aus dieser Datei

	// LTDS Dateivormat --- Dient zum speichern von Schriftarten
	class LTDSPixel
	{
	private:
		int ref;
		char index;
		char iA;
		char miA;
		char maxIndex; // Lnge des Pixels
		bool nder;
		bool nderA;
		unsigned char komp : 3; // Komprimierung der Farbwerte
		unsigned char alpha;
		LTDSPixel * davor;
		bool addBitZuFarbe( unsigned char bit ); // Fgt den Farbwerten ein Bit hinzu
		bool getNextFarbeBit( char &byte, int i ); // Speichert das nchste Farbbit in byte

	public:
		// Konstruktor 
		LTDSPixel( LTDSPixel *davor );
		// Destruktor 
		~LTDSPixel();
		// nicht constant 
		// zum Laden gedacht
		char addByte( char byte, char begin ); // gibt ende des Pixels zurck, -1 wenn nicht zu ende
		// zum speichern gedacht
		void setAlpha( unsigned char alpha ); // setzt die Farbe des Pixels
		void Komp(); // Komprimiert den Pixel
		char getNextByte( char &byte, int bbegin ); // Gibt die nchsten Bits Zurck, -1 wenn der Pixel nicht zu ende ist
		// constant 
		unsigned char getKomp() const; // hat sich die Komprimierung gendert
		bool getnderA() const;
		unsigned char getA() const; // gibt Alpha zurck
		// Reference Counting 
		LTDSPixel *getThis();
		LTDSPixel *release();
	};

	class LTDSDateiKopf
	{
	private:
		int ref;
		unsigned char sganzahl;
		unsigned char *gr;
		int *pos;

	public:
		// Konstruktor 
		LTDSDateiKopf();
		// Destruktor 
		~LTDSDateiKopf();
		// nicht constant 
		void laden( std::ifstream *inF ); // Lt aus inF
		void addSG( char sg ); // Schriftgre hinzufgen
		void removeSG( char sg ); // Schriftgre entfernen
		// constant 
		void speichern( std::ofstream *outF ) const; // Speichert nach outF
		unsigned char *getSchriftGreList() const; // gibt eine Liste mit gespeicherten Schriftgren zurck
		int *getPositionList() const; // gibt eine Positionsliste der gespeicherten Schriftgren zurck
		int getSchriftGreAnzahl() const; // gibt die Anzahl der gespeicherten Schriftgren zurck
		// Reference Counting 
		LTDSDateiKopf *getThis();
		LTDSDateiKopf *release();
	};

	class LTDSSchriftKopf
	{
	private:
		int ref;
		unsigned char schriftGre;
		unsigned char *zeichen;
		int *pos;
		unsigned char zeichenAnzahl;

	public:
		// Konstruktor 
		LTDSSchriftKopf();
		// Destruktor 
		~LTDSSchriftKopf();
		// nicht constant 
		void laden( std::ifstream *inF ); // lht von inF
		void setSchriftgre( unsigned char gr ); // setze schriftgre
		void setZeichenAlphabet( Alphabet *alphabet ); // setzt die Zeichen von alphabet
		void addZeichen( unsigned char zeichen ); // Zeichen hinzufgen
		void removeZeichen( unsigned char zeich ); // Zeichen entfernen
		// constant 
		void speichern( std::ofstream *outF ) const; // speichert nach outF
		unsigned char getSchriftGre() const; // gibt die Schriftgre zurck
		unsigned char getZeichenAnzahl() const; // gibt die Zeichenanzahl zurck
		int *getPositionen() const; // gibt die Zeichenpositionen zurck
		unsigned char *getZeichen() const; // gibt die zeichen zurck
		// Reference Counting 
		LTDSSchriftKopf *getThis();
		LTDSSchriftKopf *release();
	};

	class LTDSBuchstabenKopf
	{
	private:
		int ref;
		unsigned char zeichen;
		Punkt *gre;

	public:
		// Konstruktor 
		LTDSBuchstabenKopf();
		// Destruktor 
		~LTDSBuchstabenKopf();
		// nicht constant 
		void laden( std::ifstream *inF ); // lt aus inF
		void init( unsigned char zeichen, Punkt *gre ); // initialisierung( fr speichern )
		void init( unsigned char zeichen, int br, int h );
		// constant 
		void speichern( std::ofstream *outF ) const; // speichertn nach outF
		unsigned char getZeichen() const; // gibt das Zeichen zurck
		int getBreite() const; // gibt die Breite zurck
		int getHhe() const; // gibt die hhe zurck
		Punkt *getGre() const; // gibt die Gre zurck
		// Reference Counting 
		LTDSBuchstabenKopf *getThis();
		LTDSBuchstabenKopf *release();
	};

	class LTDSBuchstabenKrper
	{
	private:
		int ref;
		Punkt *gre;
		unsigned char zeichen;
		Buchstabe *buchstabe;

	public:
		// Konstruktor 
		LTDSBuchstabenKrper( LTDSBuchstabenKopf *kopf );
		// Destruktor 
		~LTDSBuchstabenKrper();
		// nicht constant 
		void setBuchstabe( Buchstabe *zeichen ); // setzt den Buchstaben
		void laden( std::ifstream *inF ); // Lht aus inF
		// constant 
		void speichern( std::ofstream *outF ) const; // speichert nach outF
		Buchstabe *getBuchstabe() const; // gibt den Buchstaben zurck
		unsigned char getZeichen() const; // gibt das Zeichen zurck
		// Reference Counting 
		LTDSBuchstabenKrper *getThis();
		LTDSBuchstabenKrper *release();
	};

	class LTDSDatei
	{
	private:
		int ref;
		Text *pfad;
		LTDSDateiKopf *dateiKopf;
		CRITICAL_SECTION thread_Save;

	public:
		// Konstruktor 
		LTDSDatei();
		// Destruktor 
		~LTDSDatei();
		// nicht constant 
		void lockDatei();
		void unlockDatei();
		void setPfad( Text *txt ); // setzt den Pfad zur Datei
		void leseDaten(); // liet den Dateikopf
		void addSchriftgre( Alphabet *alphabet ); // fgt eine Schriftgre hinzu
		void addBuchstabe( int gr, Buchstabe *zeich, unsigned char zeichen ); // Fgt einer Schriftgre einen Buchstaben hinzu
		void lscheSchrifrGre( int gr ); // Lscht eine Schriftgre aus der Datei
		void lscheBuchstabe( int gr, unsigned char zeichen ); // Lscht einen Buchstaben aus der Datei
		void lscheDatei(); // Lscht die gesamte Datei
		void erstelleDatei(); // erstellt die Datei
		void speicherSchrift( Schrift *schrift ); // Speichert die bergebene Schrift
		// constant 
		Schrift *ladeSchrift(); // gibt die geladene Schrift zurck
		Alphabet *ladeAlphabet( int schriftgre ); // gibt eine geladene Schrift nur mit der angegebenen Schriftgre zurck
		Buchstabe *ladeBuchstabe( int schriftgre, unsigned char zeichen ); // Ld einen bestimmten Buchstaben
		Text *getPfad() const; // gibt den Dateipfad zurck
		int getAnzahlSchriftgren() const; // gibt die Anzahl der Schriftgren aus der Datei zurck
		unsigned char *getSchriftGren() const; // gibt einen Array von Schriftgren zurck
		unsigned char getAnzahlBuchstaben( int sg ); // gibt die anzahl gespeicherter Buchstaben einer Schriftgre zurck
		unsigned char *getBuchstaben( int sg ); // gibt einen Array von Buchstaben einer Schriftgre zurck
		// Reference Counting 
		LTDSDatei *getThis();
		LTDSDatei *release();
	};

	// Bit Funktionen 
	int Bits( int a ); // gibt 1-bits in gewinschter anzahl zurck
	int getBits( char c ); // gibt zurck, wie viele Bits c bentigt
}
#endif