#ifndef Schrift_H
#define Schrift_H

#define WIN32_LEAN_AND_MEAN
#include <Windows.h>

namespace Framework
{
	class Farbe; // Farbe.h
	class Punkt; // Punkt.h
	class Bild; // Bild.h
	class Text; // Text.h
	class Buchstabe; // aus dieser Datei
	class Alphabet; // aus dieser Datei
	class Schrift; // aus dieser Datei

	class Buchstabe
	{
	private:
		Punkt *gre;
		Punkt *pos;
		unsigned char *alpha;
		Farbe *f;
		int schriftGre;
		int drawSg;
		int ref;

	public:
		// Konstruktor 
		Buchstabe();
		// Destruktor 
		~Buchstabe();
		// nicht constant 
		void NeuBuchstabe( Punkt *gre ); // Initialisierung
		void setPixel( Punkt *pos, unsigned char alpha ); // setzt den alphawert des Pixels
		void setPixel( int x, int y, unsigned char alpha );
		void setPixel( int i, unsigned char alpha );
		void setPosition( Punkt *pos ); // setzt die Buchstabenposition
		void setPosition( int x, int y );
		void setFarbe( int fc ); // setzt die Buchstabenfarbe
		void setFarbe( Farbe *f );
		void setSchriftGre( int sg ); // setzt die Schriftgre des Buchstaben
		void setDrawSchriftGre( int dsg ); // setzt die Zeichengre des Buchstaben
		// constant 
		unsigned char *getBuff() const;
		Farbe *getFarbe() const; // gibt die Buchstabenfarbe zurck
		Punkt *getGre() const; // gibt die Buchstabenbildgre zurck
		int getBreite() const; // Buchstabenbreite
		int getHhe() const; // Buchstabenhhe
		void render( Bild *zrObj ) const; // Zeichnet nach rObj
		// Reference Counting 
		Buchstabe *getThis();
		Buchstabe *release();
	};

	class Alphabet
	{
	private:
		Buchstabe **zeichen;
		int schriftGre;
		int drawSchriftGre;
		Punkt *pos;
		int zeilenHhe;
		int zeilenAbstand;
		Farbe *f;
		int ref;

	public:
		// Konstruktor 
		Alphabet();
		// Destruktor 
		~Alphabet();
		// nicht constant 
		void NeuAlphabet(); // Initialisierung 
		void setBuchstabe( unsigned char i, Buchstabe *buchstabe ); // setzt einen Buchstaben
		void setSchriftgre( int gr ); // setzt die Schriftgre
		void setDrawSchriftgre( int gr ); // setzt die Zeichengre
		void setZeilenAbstand( int za ); // setzt den Zeilenabstand
		void setDrawPosition( Punkt *pos ); // setzt die Draw Position
		void setDrawPosition( int x, int y );
		void setFarbe( Farbe *f ); // setzt die Schriftfarbe
		void setFarbe( int fc );
		// constant 
		Buchstabe *getBuchstabe( unsigned char i ) const; // gibt einen Buchstaben zurck
		int getSchriftgre() const; // gibt die Schriftgre zurck
		int getDrawSchriftGre() const; // gibt die Zeichengre zurck
		int getZeilenAbstand() const; // gibt den Zeilenabstand zurck
		int getZeilenHhe() const; // gibt die Hhe des hchsten Zeichens zurck
		Punkt *getPosition() const; // gibt die DrawPosition zurck
		Farbe *getFarbe() const; // gibt die Farbe zurck
		int getTextBreite( Text *txt ) const; // gibt die Breite des Textes zurck
		int getTextHhe( Text *txt ) const; // gibt die Hhe des Textes zurck
		int textPos( Text *txt, int mausX, int mausY ) const; // gibt den Buchstaben zurck, auf den die Maus zeigt
		void render( Text *txt, Bild *zrObj ) const; // Zeichnet txt nach renderObj
		void render( int xOff, int yOff, int bOff, int hOff, Text *txt, Bild *zrObj ) const;
		void render( Text *txt, Bild *zrObj, int cpos, int cf, int fbeg, int ff ) const;
		void render( int xOff, int yOff, int bOff, int hOff, Text *txt, Bild *zrObj, int cpos, int cf, int fbeg, int ff ) const;
		// Reference Counting 
		Alphabet *getThis();
		Alphabet *release();
	};

	class AlphabetArray
	{
	private:
		AlphabetArray *next;
		Alphabet *This;

	public:
		// Konstruktor 
		AlphabetArray();
		// Destruktor 
		~AlphabetArray();
		// nicht constant 
		void addAlphabet( Alphabet *alphabet ); // Fgt ein Alphabet hinzu
		bool removeAlphabet( int sg ); // entfernt ein Alphabet
		void setDrawSchriftGre( int sg ); // Setzt die Draw Schriftgre aller Alphabete
		void setZeilenAbstand( int za ); // setzt den Zeilenabstant aller Alphabete
		void setSchriftFarbe( Farbe *farbe ); // setzt die Schriftfarbe aller Alphabete
		void setSchriftFarbe( int fc );
		void setNext0(); // setzt den next Zeiger zu 0
		// constant 
		Alphabet *getAlphabet( unsigned char sg ) const; // gibt getThis von einem Alphabet zurck
		Alphabet *zAlphabet( unsigned char sg ) const; // gibt ein Alphabet zurck
		Alphabet *getAlphabetI( int index, int count ) const;
		AlphabetArray *getNext() const; // gibt das nchste Alphabet zurck
	};

	class Schrift
	{
	private:
		unsigned char alphabetAnzahl;
		AlphabetArray *alphabet;
		int schriftGre;
		int zeilenAbstand;
		Punkt *drawPos;
		Farbe *farbe;
		CRITICAL_SECTION threadSave;
		int ref;

	public:
		// Konstruktor 
		Schrift();
		// Destruktor 
		~Schrift();
		// nicht constant 
		void addAlphabet( Alphabet *alphabet ); // Fgt der Schrift ein Alphabet hinzu
		void removeAlphabet( int sg ); // Entfernt ein Alphabet
		void setDrawPosition( int x, int y ); // setzt die Zeichenposition
		void setDrawPosition( Punkt *pos );
		void setSchriftGre( int sg ); // setzt die Schriftgre
		void setZeilenAbstand( int za ); // setzt den Zeilenabstand
		void setFarbe( Farbe *f ); // setzt die Schriftfarbe
		void setFarbe( int fc );
		void renderText( Text *txt, Bild *zrObj ); // zeichnet txt nach rObj
		void renderText( int xOff, int yOff, int bOff, int hOff, Text *txt, Bild *zrObj );
		void renderText( Text *txt, Bild *zrObj, int cpos, int cf, int fbeg, int ff );
		void renderText( int xOff, int yOff, int bOff, int hOff, Text *txt, Bild *zrObj, int cpos, int cf, int fbeg, int ff );
		void lockSchrift(); // blockiert die Schrift fr andere Threads
		void unlockSchrift(); // gibt die Schrift fr andere Threads frei
		// constant 
		Alphabet *getAlphabet( int sg ) const; // gibt einen Alphaberarray zurck
		Alphabet *getAlphabetI( int index ) const;
		unsigned char getAlphabetAnzahl() const; // gibt die anzahl von in der Schrift enthaltenen Alphabeten zurck
		int getSchriftGre() const; // gibt die Schriftgre zurck
		int getZeilenabstand() const; // gibt den Zeilenabstand zurck
		int getDrawX() const; // gibt die Zeichenposition zurck
		int getDrawY() const;
		Punkt *getDrawPosition() const;
		Farbe *getFarbe() const; // gibt die Farbe zurck
		unsigned int getFarbeCode() const;
		int getTextBreite( Text *txt ) const; // gibt die Breite des Textes zurck
		int getTextHhe( Text *txt ) const; // gibt die Hhe des Textes zurck
		int textPos( Text *txt, int mausX, int mausY ) const; // gibt den Buchstaben zurck, auf den die Maus zeigt
		// Reference Counting 
		Schrift *getThis();
		Schrift *release();
	};
}

#endif